/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/calendar/gregorian/gregorian_arithmetic.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef gregorian_arithmetic
#define gregorian_arithmetic
/* arithmetic.h
*
*  This module contains all variable span procedures for
*  variable span arithmetic.
*
*/

/*  
*  Routine:  cal_standard_gregorian_v1_neg_s
*
*  Description: Return a negative span whose duration is the
*		same as the input span
*
*  Arguments: 	spanin -- (IN) : span to be converted
*               result --(OUT): result negative span
*
*  Return Value: Error code
*
*  Errors: None
*  
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_neg_s(span_type spanin, span_type result);

/*
*  Routine:  cal_standard_gregorian_v1_vs_add_fs
*
*  Description: Compute the sum of a variable span and a fixed span
*
*  Arguments:   vs    --   (IN) : variable span
* 		fs    --   (IN) : fixed span
*               result--   (OUT): the sum of the arguments
*
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_add_fs(span_type vs, span_type fs, span_type result);

/*
*  Routine:  cal_standard_gregorian_v1_vs_add_vs
*
*  Description: Compute the sum of two variable spans
*
*  Arguments:   vs1, vs2 -- (IN) : spans to be added
*               result   -- (OUT): result of sum of two spans
*
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_add_vs(span_type vs1, span_type vs2, span_type result);

/*
*  Routine:  cal_standard_gregorian_v1_vs_minus_fs
*
*  Description: Subtract a fixed span from a variable span
*
*  Arguments:   vs    --      (IN) : variable span
* 		fs    --      (IN) : fixed span
*               result--      (OUT): the difference of the arguments
*
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_minus_fs(span_type vs, span_type fs, span_type result);

/*
*  Routine:  cal_standard_gregorian_v1_fs_minus_vs
*
*  Description: Subtract a fixed span from a variable span
*
*  Arguments:   fs    --  (IN) : fixed span
* 		vs    --  (IN) : variable span
*               result--  (OUT): the difference of the arguments
*
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_fs_minus_vs(span_type fs, span_type vs, span_type result);

/*
*  Routine:  cal_standard_gregorian_v1_vs_minus_vs
*
*  Description: Subtract a variable span from a variable span
*
*  Arguments:   vs1   --  (IN) : variable span
* 		vs2   --  (IN) : variable span 
*               result--  (OUT): the difference of the arguments
* 
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_minus_vs(span_type vs1, span_type vs2, span_type result);
 
/*
*  Routine:  cal_standard_gregorian_v1_vs_add_e
*
*  Description:  Add a variable span to an event
*
*  Arguments: vs        --  (IN) : variable span
*             event_in  --  (IN) : event
*             result    --  (OUT): the sum of the arguments
*
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_add_e(span_type vs, event_type event_in, event_type result);

/*
*  Routine:  cal_standard_gregorian_v1_e_minus_vs
*
*  Description:  Subtract a variable span from an event
*
*  Arguments: event_in  --  (IN) : event 
*             vs        --  (IN) : variable span
*             result    --  (OUT): the difference of the arguments

*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_e_minus_vs(event_type event_in, span_type vs, event_type result);

/*
*  Routine:  cal_standard_gregorian_v1_vs_times_n
*
*  Description:  Multiply a variable span by a numeric value
*
*  Arguments:   vs    --  (IN) : variable span
*               n     --  (IN) : numeric value
*               result--  (OUT): the product of the arguments
*
*  Return Value: Error code
*
*  Errors: out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_times_n(span_type vs, double n, span_type result);

/*
*  Routine:  cal_standard_gregorian_v1_vs_div_n
*
*  Description:  Divide a variable span by a numeric value
*
*  Arguments: vs     --    (IN) : variable span
*             n      --    (IN) : numeric value
*             result --    (OUT): the division of vs by n
*
*  Return Value: Error code
*
*  Errors: None (assumes that UCS checks that n is not 0)
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_div_n(span_type vs, double n, span_type result);

/*
*  Routine:  cal_standard_gregorian_v1_fs_div_vs
*
*  Description:  Divide a fixed span by a variable span
*
*  Arguments:   fs        -- (IN) : fixed span dividend
*               vs        -- (IN) : variable span divisor
*               dividend  -- (OUT): the division of fs by vs
*
*  Return Value:  Error code
*
*  Errors: divide_by_zero
*
*  Side Effects: Overwrites dividend
*/

error_type cal_standard_gregorian_v1_fs_div_vs(span_type fs, span_type vs, double *dividend);

/*
*  Routine:  cal_standard_gregorian_v1_vs_div_fs
*
*  Description:  Divide a variable span by a fixed span
*
*  Arguments:   vs        -- (IN) : variable span dividend
*               fs        -- (IN) : fixed span divisor
*               dividend  -- (OUT): the division of vs by fs
*
*  Return Value: Error code
*
*  Errors: None (assume that UCS checks to make sure fs is not 0)
*
*  Side Effects: Overwrites dividend
*/

error_type cal_standard_gregorian_v1_vs_div_fs(span_type vs, span_type fs, double *dividend);

/*
*  Routine:  cal_standard_gregorian_v1_vs_div_vs
*
*  Description:  Divide a variable span by a variable span
*
*  Arguments:   vs1      -- (IN) : variable span dividend
*               vs2      -- (IN) : variable span divisor
*               dividend -- (OUT): the division of vs1 by vs2
*
*  Return Value: Error code
*
*  Errors: semantic_error
*
*  Side Effects: Overwrites dividend
*/

error_type cal_standard_gregorian_v1_vs_div_vs(span_type vs1, span_type vs2, double *dividend);

/*
*  Routine:  cal_standard_gregorian_v1_i_add_vs
*
*  Description:  Add a variable span to an interval
*
*  Arguments: interval_in  -- (IN) : interval 
*             vs           -- (IN) : variable span 
*             result       -- (OUT): the sum of the arguments
*
*  Return Value: Error code
*
*  Errors:  out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_vs_add_i(span_type vs, interval_type interval_in, interval_type result);

/*
*  Routine:  cal_standard_gregorian_v1_i_minus_vs
*
*  Description:  Subtract a variable span from an interval
*
*  Arguments: interval_in  --  (IN) : the interval to be displaced
*             vs           --  (IN) : the span of displacement
*             result       --  (OUT): the difference of the arguments
*
*  Return Value: Error code
*
*  Errors:  out_of_range
*
*  Side Effects: Overwrites result
*/

error_type cal_standard_gregorian_v1_i_minus_vs(interval_type interval_in, span_type vs, interval_type result);
#endif
