/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/tadti_builtin.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 

#ifdef INDETERMINATE

#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include "system_wide.h"
#include "tadt.h"

extern int sizes[MAX_RESOLUTIONS_OF_EVENTS][MAX_PRECISION];
extern context_type context;

/* 
* Routine: tadti_intersect
*  
* Description: Returns the indeterminate overlap of two intervals, if
*	       they intersect.  Otherwise leave the result
*              unchanged.
*
* Arguments:  gamma -- (IN): plausibility 
*             i1, i2 -- (IN): intervals for intersection
*             result -- (OUT): result interval if intervals intersect 
*
* Return Value: Error code
* 
* Errors: tadt_invalid_interval
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadti_intersect(int gamma, interval_type i1, interval_type i2, interval_type result)
{
  if (tadti_i_overlaps_i(gamma,i1,i2)) {
    tadti_first(gamma,i1->terminate,i2->terminate,result->terminate);
    tadti_last(gamma,i1->start,i2->start,result->start);
    return tadt_ok;
    }
  return tadt_invalid_interval;
}

/* 
*  Routine: tadti_first
*  
*  Description: Returns the earlier of two events using plausibility
*
* Arguments:  gamma -- (IN): plausibility 
*             event1, event2 -- (IN): two input events
*             result -- (OUT) earliest event.
*
*  Return Value: Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadti_first(int gamma, event_type event1, event_type event2, event_type result)
{
  bool c1, c2;
  event_type tempev;

  c1 = tadti_e_precedes_e(gamma, event1,event2);
  c2 = tadti_e_precedes_e(gamma, event2,event1);
  if (c1) 
    if (c2) {
      /* indistinguishable_set_merge(event1, event2, result); */

      /* Set result to indistinguishable type */
      result->generic.ts_type = indistinguishable;

      /* Copy event1 to first element position */
      tadt_copy_e(event1, (event_type)&(result->indistinguishable.element));

      /* Allocate a new event and link it into the set */
      tempev = quickly_allocate_event();
      result->indistinguishable.next = tempev;
      tempev->generic.ts_type = indistinguishable;
      tempev->indistinguishable.next = NULL;
      tadt_copy_e(event2, (event_type)&(tempev->indistinguishable.element));
      }
    else 
      tadt_copy_e(event1,result);
  else
    if (c2)
      tadt_copy_e(event2,result);
    else {
      /* incomparable_set_merge(event1, event2, result); */

      /* Set result to incomparable type */
      result->generic.ts_type = incomparable;

      /* Copy event1 to first element position */
      tadt_copy_e(event1, (event_type)&(result->incomparable.element));

      /* Allocate a new event and link it into the set */
      tempev = quickly_allocate_event();
      result->incomparable.next = tempev;
      tempev->generic.ts_type = incomparable;
      tempev->incomparable.next = NULL;
      tadt_copy_e(event2, (event_type)&(tempev->incomparable.element));

      }

  return tadt_ok;
}

/* 
*  Routine: tadti_last
*  
*  Description: Returns the later event of two events using plausibility
*
* Arguments:  gamma -- (IN): plausibility 
*             event1, event2 -- (IN): two input events
*             result -- (OUT) earliest event.
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadti_last(int gamma, event_type event1, event_type event2, event_type result)
{
  if (tadti_e_precedes_e(gamma,event1,event2)) 
     tadt_copy_e(event2,result);
  else tadt_copy_e(event1,result);
  return tadt_ok;
}

void tadti_shrink_s(int amount, event_type event, event_type result) 
{
   int temp32, temp_start32, temp_finish32;
   event_type lower, upper;

   if (event != result) tadt_copy_e(event,result);

   /* Nonstandard Types? */
   if ((context->resolution == packed_chunked_nonstandard) ||
       (context->resolution == packed_general_nonstandard)) {
      
     /* Event has been unpacked previously */
     /* Assume 32 bit stuff here */
     temp32 = event->upack_ind.distribution.left_off;
     temp32 += (((100 - temp32) * amount)/100);
     result->upack_ind.distribution.left_off = temp32;
     lower = (event_type)&event->upack_ind.lower;
     upper = (event_type)&event->upack_ind.upper;
     temp_start32 = lower->upack32.granules;
     temp_finish32 = upper->upack32.granules;
     lower = (event_type)&result->upack_ind.lower;
     lower->upack32.granules = 
                temp_start32 + ((temp_finish32 - temp_start32) * amount/100);
   }
}

void tadti_shrink_t(int amount, event_type event, event_type result) 
{
   int temp32, temp_start32, temp_finish32;
   event_type lower, upper;

   if (event != result) tadt_copy_e(event,result);

   /* Nonstandard Types? */
   if ((context->resolution == packed_chunked_nonstandard) ||
       (context->resolution == packed_general_nonstandard)) {
      
     /* Event has been unpacked previously */
     /* Assume 32 bit stuff here */
     temp32 = event->upack_ind.distribution.right_off;
     temp32 += (((100 - temp32) * amount)/100);
     result->upack_ind.distribution.right_off = temp32;
     lower = (event_type)&event->upack_ind.lower;
     upper = (event_type)&event->upack_ind.upper;
     temp_start32 = lower->upack32.granules;
     temp_finish32 = upper->upack32.granules;
     upper = (event_type)&result->upack_ind.upper;
     upper->upack32.granules = 
                temp_finish32 - ((temp_finish32 - temp_start32) * amount/100);
   }
}

/* OUT
static int adjust_search(int high, int low, event_type result1, event_type result2)
{
   int median;
   event_space_type temp1, temp2;
   event_type upper, lower;
   
   tadt_copy_e(result1, &temp1);
   tadt_copy_e(result2, &temp2);

   median = low + (high - low)/2;
   if (median == low) return median;

   tadti_shrink_t(median, &temp1, &temp1);
   tadti_shrink_s(median, &temp2, &temp2);
   upper = (event_type)&(temp1.upack_ind.upper);
   lower = (event_type)&(temp2.upack_ind.lower);
   if (upper->upack32.granules <= lower->upack32.granules)
     return adjust_search(median, low, result1, result2);
   else 
     return adjust_search(high, median, result1, result2);
}

bool tadti_adjust(int gamma, interval_type interval, interval_type result)
{
   int i, adjust_search();

   if (interval != result) tadt_copy_i(interval,result);

   i = adjust_search(gamma/2, 0, interval->start, interval->terminate);

   tadti_shrink_t(i, result->start, result->start);
   tadti_shrink_s(i, result->terminate, result->terminate);

   return TRUE;  
}
*/

#endif
